@@ -1,35 +1,31 @@
 /*
- * Copyright (c) 2003-2004 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2003-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/*
+ * Read/write patterns (default is always "forward")
+ */
+#define IO_RANDOM	( 0)
+#define IO_FORWARD	( 1)
+#define IO_BACKWARD	(-1)
+
+/*
+ * File descriptor options
+ */
 #define IO_READONLY	(1<<0)
 #define IO_DIRECT	(1<<1)
 #define IO_REALTIME	(1<<2)
@@ -38,6 +34,7 @@
 #define IO_CREAT	(1<<5)
 #define IO_TRUNC	(1<<6)
 #define IO_FOREIGN	(1<<7)
+#define IO_NONBLOCK	(1<<8)
 
 /*
  * Regular file I/O control
@@ -80,9 +77,13 @@ extern int		openfile(char *, xfs_fsop_geom_t *, int, mode_t);
 extern int		addfile(char *, int , xfs_fsop_geom_t *, int);
 extern void		printxattr(uint, int, int, const char *, int, int);
 
+extern unsigned int	recurse_all;
+extern unsigned int	recurse_dir;
+
 extern void		*buffer;
-extern ssize_t		buffersize;
-extern int		alloc_buffer(ssize_t, int, unsigned int);
+extern size_t		buffersize;
+extern int		align_direct(off64_t *, long long *);
+extern int		alloc_buffer(size_t, int, unsigned int);
 extern int		read_buffer(int, off64_t, long long, long long *,
 					int, int);
 extern void		dump_buffer(off64_t, ssize_t);
@@ -92,7 +93,9 @@ extern void		bmap_init(void);
 extern void		file_init(void);
 extern void		freeze_init(void);
 extern void		fsync_init(void);
+extern void		getrusage_init(void);
 extern void		help_init(void);
+extern void		imap_init(void);
 extern void		inject_init(void);
 extern void		mmap_init(void);
 extern void		open_init(void);
@@ -100,6 +103,7 @@ extern void		pread_init(void);
 extern void		prealloc_init(void);
 extern void		pwrite_init(void);
 extern void		quit_init(void);
+extern void		shutdown_init(void);
 extern void		truncate_init(void);
 
 #ifdef HAVE_FADVISE
@@ -126,12 +130,6 @@ extern void		sendfile_init(void);
 #define sendfile_init()	do { } while (0)
 #endif
 
-#ifdef HAVE_SHUTDOWN
-extern void		shutdown_init(void);
-#else
-#define shutdown_init()	do { } while (0)
-#endif
-
 #ifdef HAVE_MADVISE
 extern void		madvise_init(void);
 #else
@@ -144,3 +142,8 @@ extern void		mincore_init(void);
 #define mincore_init()	do { } while (0)
 #endif
 
+#ifdef HAVE_PARENT
+extern void		parent_init(void);
+#else
+#define parent_init()	do { } while (0)
+#endif
